import { TextChannel } from 'eris';
import { v4 as genUUID } from 'uuid';
import { RichEmbed, Route, Server } from '../../../class';

export default class Root extends Route {
  constructor(server: Server) {
    super(server);
    this.conf = {
      path: '/',
    };
  }

  public bind() {
    this.router.post('/eo', async (req, res) => {
      if (!req.body.pin) {
        return res.status(401).json({
          code: this.constants.codes.UNAUTHORIZED,
          message: this.constants.messages.UNAUTHORIZED,
        });
      }

      const director = await this.server.client.db.Score.findOne({ pin: req.body.pin });
      const staffGuild = this.server.client.guilds.get('446067825673633794') || await this.server.client.getRESTGuild('446067825673633794');

      if (!director || !staffGuild.members.get(director.userID)?.roles?.includes('662163685439045632')) {
        return res.status(401).json({
          code: this.constants.codes.UNAUTHORIZED,
          message: this.constants.messages.UNAUTHORIZED,
        });
      }

      if (!req.body.subject || !req.body.body) {
        return res.status(400).json({
          code: this.constants.codes.CLIENT_ERROR,
          message: this.constants.messages.CLIENT_ERROR,
        });
      }

      const executiveOrder = await this.server.client.db.ExecutiveOrder.create({
        issuedBy: director.userID,
        subject: req.body.subject,
        body: req.body.body,
        at: new Date(),
        oID: genUUID(),
      });

      const staffDiscord = this.server.client.users.get(director.userID) || await this.server.client.getRESTUser(director.userID);
      const staffInformation = await this.server.client.db.Staff.findOne({ userID: director.userID });

      const embed = new RichEmbed();
      embed.setTitle('Executive Order');
      embed.setAuthor(`${staffDiscord.username}#${staffDiscord.discriminator}, ${staffInformation.pn.join(', ')}`, staffDiscord.avatarURL);
      embed.setColor('#dd3acd');
      embed.addField('Subject', req.body.subject);
      embed.addField('Body', req.body.body);
      embed.setDescription(executiveOrder.oID);
      embed.setTimestamp(new Date());

      const channel = <TextChannel>this.server.client.getChannel('807444198969835550');
      await channel.createMessage({ embed });

      res.status(200).json({
        code: this.constants.codes.SUCCESS,
        message: `Created new Executive Order with ID ${executiveOrder.oID} by ${staffDiscord.username}#${staffDiscord.discriminator}, ${staffInformation.pn.join(', ')}.`,
      });
    });

    this.router.post('/motion', async (req, res) => {
      if (!req.body.pin) {
        return res.status(401).json({
          code: this.constants.codes.UNAUTHORIZED,
          message: this.constants.messages.UNAUTHORIZED,
        });
      }

      const director = await this.server.client.db.Score.findOne({ pin: req.body.pin });
      const staffGuild = this.server.client.guilds.get('446067825673633794') || await this.server.client.getRESTGuild('446067825673633794');

      if (!director || !staffGuild.members.get(director.userID)?.roles?.includes('662163685439045632')) {
        return res.status(401).json({
          code: this.constants.codes.UNAUTHORIZED,
          message: this.constants.messages.UNAUTHORIZED,
        });
      }

      if (!req.body.subject || !req.body.body) {
        return res.status(400).json({
          code: this.constants.codes.CLIENT_ERROR,
          message: this.constants.messages.CLIENT_ERROR,
        });
      }

      const motionID = genUUID();

      const staffDiscord = this.server.client.users.get(director.userID) || await this.server.client.getRESTUser(director.userID);
      const staffInformation = await this.server.client.db.Staff.findOne({ userID: director.userID });

      const embed = new RichEmbed();
      embed.setTitle('Motion');
      embed.setAuthor(`${staffDiscord.username}#${staffDiscord.discriminator}, ${staffInformation.pn.join(', ')}`, staffDiscord.avatarURL);
      embed.setColor('#66e1ff');
      embed.addField('Subject', req.body.subject);
      embed.addField('Body', req.body.body);
      embed.setDescription(motionID);
      embed.setTimestamp(new Date());

      const channel = <TextChannel>this.server.client.getChannel('807444198969835550');
      const motionMessage = await channel.createMessage({ embed });
      await motionMessage.addReaction(this.server.client.util.emojis.SUCCESS);
      await motionMessage.addReaction(this.server.client.util.emojis.ERROR);

      const motion = await this.server.client.db.Motion.create({
        issuedBy: director.userID,
        subject: req.body.subject,
        body: req.body.body,
        at: new Date(),
        oID: motionID,
        motionMessage: motionMessage.id,
        processed: false,
      });

      res.status(200).json({
        code: this.constants.codes.SUCCESS,
        message: `Created new Motion with ID ${motion.oID} by ${staffDiscord.username}#${staffDiscord.discriminator}, ${staffInformation.pn.join(', ')}.`,
      });
    });

    this.router.delete('/eo/:id', async (req, res) => {
      if (!req.params.id) return res.status(400).send(this.constants.messages.CLIENT_ERROR);
      if (!(await this.server.client.db.ExecutiveOrder.exists({ oID: req.params.id }))) return res.status(404).send(this.constants.messages.NOT_FOUND);

      await this.server.client.db.ExecutiveOrder.deleteOne({ oID: req.params.id });

      res.status(200).send({ message: `Executive Order with ID ${req.params.id} deleted.` });
    });

    this.router.delete('/motion/:id', async (req, res) => {
      if (!req.params.id) return res.status(400).send(this.constants.messages.CLIENT_ERROR);
      if (!(await this.server.client.db.Motion.exists({ oID: req.params.id }))) return res.status(404).send(this.constants.messages.NOT_FOUND);

      await this.server.client.db.Motion.deleteOne({ oID: req.params.id });

      res.status(200).send({ message: `Motion with ID ${req.params.id} deleted.` });
    });

    this.router.get('/eo/:id', async (req, res) => {
      if (!req.params.id) return res.status(400).send(this.constants.messages.CLIENT_ERROR);
      if (!(await this.server.client.db.ExecutiveOrder.exists({ oID: req.params.id }))) return res.status(404).send(this.constants.messages.NOT_FOUND);

      const executiveOrder = await this.server.client.db.ExecutiveOrder.findOne({ oID: req.params.id });

      res.status(200).send({
        issuedBy: executiveOrder.issuedBy,
        id: executiveOrder.oID,
        subject: executiveOrder.subject,
        body: executiveOrder.body,
        at: new Date(executiveOrder.at),
      });
    });

    this.router.get('/motion/:id', async (req, res) => {
      if (!req.params.id) return res.status(400).send(this.constants.messages.CLIENT_ERROR);
      if (!(await this.server.client.db.Motion.exists({ oID: req.params.id }))) return res.status(404).send(this.constants.messages.NOT_FOUND);

      const motion = await this.server.client.db.Motion.findOne({ oID: req.params.id });

      res.status(200).send({
        issuedBy: motion.issuedBy,
        id: motion.oID,
        subject: motion.subject,
        body: motion.body,
        at: new Date(motion.at),
      });
    });

    this.router.patch('/eo/:id', async (req, res) => {
      if (!req.params.id) return res.status(400).send(this.constants.messages.CLIENT_ERROR);
      if (!(await this.server.client.db.ExecutiveOrder.exists({ oID: req.params.id }))) return res.status(404).send(this.constants.messages.NOT_FOUND);
      if (!req.body.subject && !req.body.body) return res.status(400).send(this.constants.messages.CLIENT_ERROR);

      const executiveOrder = await this.server.client.db.ExecutiveOrder.findOne({ oID: req.params.id });
      await executiveOrder.updateOne({
        subject: req.body.subject || executiveOrder.subject,
        body: req.body.body || executiveOrder.body,
      });

      res.status(200).send({ message: `Updated Executive Order with ID ${executiveOrder.oID}.` });
    });

    this.router.patch('/motion/:id', async (req, res) => {
      if (!req.params.id) return res.status(400).send(this.constants.messages.CLIENT_ERROR);
      if (!(await this.server.client.db.Motion.exists({ oID: req.params.id }))) return res.status(404).send(this.constants.messages.NOT_FOUND);
      if (!req.body.subject && !req.body.body) return res.status(400).send(this.constants.messages.CLIENT_ERROR);

      const motion = await this.server.client.db.Motion.findOne({ oID: req.params.id });
      await motion.updateOne({
        subject: req.body.subject || motion.subject,
        body: req.body.body || motion.body,
      });

      res.status(200).send({ message: `Updated Motion with ID ${motion.oID}.` });
    });

    this.router.get('/motion', async (_req, res) => {
      const motions = await this.server.client.db.Motion.find().lean();

      res.status(200).send({
        motions,
      });
    });

    this.router.get('/resolution', async (_req, res) => {
      const resolutions = await this.server.client.db.Resolution.find().lean();

      res.status(200).send({
        resolutions,
      });
    });

    this.router.get('/eo', async (_req, res) => {
      const executiveOrders = await this.server.client.db.ExecutiveOrder.find().lean();

      res.status(200).send({
        executiveOrders,
      });
    });
  }
}
